#include <formatio.h>
#include <visa.h>
#include <stdlib.h>  
#include <stdio.h>  
#include <math.h>
#include <string.h>
#include "AVR3x61.h"

#define AVR3x61_REVISION  "Rev 1.0, 04/99, CVI 5.0.1"  /*Instrument driver revision*/
#define BUFFER_SIZE    512L  /* File I/O buffer size */

/*========= ADVANTEST R3x61 Series Spectrum Analyzers: R3261 A/AN/B =========*/ 
/*                                                     R3361 A/AN/B          */
/*  										                     			 */
/* LabWindows/CVI 5.0.1 Instrument Driver                                    */
/* Original Release: April 1999                                              */
/* By: Denis BIRINGER, Advantest Europe R&D S.A.R.L. Les ULIS France         */
/*     Fax + 33 1 69 18 25 90                                                */
/*     e-mail: d.biringer@erd.advantest.de                                   */
/*                                                                           */
/* Originally written in C                                                   */
/* Modification History: None                                                */
/*                                                                           */
/*===========================================================================*/
 
/*****************************************************************************/
/* INSTRUMENT-DEPENDENT COMMAND ARRAYS                                       */
/*****************************************************************************/
static ViString s_init[] = {"DL0","HD0","TYP?",""};

static ViString s_freqType[] = {"CF","FA","FB"};         
static ViString s_spanType[] = {"LS","SP","FS","ZS"}; 
static ViString s_logSpan[] = {"LGA10KZ","LGA100KZ","LGA1MZ","LGA10MZ","LGA100MZ","LGB"};
static ViString s_freqOffset[] = {"FOF","FON"};
static ViString s_freqStepSize[] = {"CS","CA"};
static ViString s_counterResolution[] = {"CN0","CN1","CN2","CN3","CNF"};
static ViString s_counterONOFF[] = {"CNF","CNN","CNF"};

static ViString s_readFreq[]= {"CF?","SP?","FA?","FB?","FO?"};
static ViString s_readLogSpan[] = {"LGA?","LGB?"};
static ViString s_readFreqStepSize[] = {"CS?","CA?"};
static ViString s_readFreqCounter[] = {"CN?","MF?"};

static ViString s_refLevel[] = {"RL"};
static ViString s_refUnit[] = {"UB","UM","UU","UE","UW"};
static ViString s_yScale[] = {"DD10DB","DD5DB","DD2DB","DD1DB","DD0.5DB","DD0.2DB","DD0.1DB",
                              "LL1","LL2","LL4","LL8","DD?","LL?"};
static ViString s_refLevelOffset[] = {"ROF","RON"};  
static ViString s_att[] = {"AT","AA"}; 
                              
static ViString s_readRefLevel[]={"RL?"};
static ViString s_readRefUnit[]={"UN?"};
static ViString s_readUnitName[] = {"dBm","dBmV","dBV","dBVemf","dBpW","dB","V"};                              
static ViString s_readScale[] = {"10dB/","5dB/","2dB/","1dB/","0.5dB/","0.2dB/","0.1dB/",
                                 "Lin x1","Lin x1.6","Lin x4","Lin x8"};                              
static ViString s_readRefLevelOffset[]={"RO?"};                                    
static ViString s_readAtt[] = {"AT?","AA?"};                              
                              
static ViString s_RBW[] = {"BA","RB30HZ","RB100HZ","RB300HZ","RB1KZ","RB3KZ","RB10KZ",
                           "RB30KZ","RB100KZ","RB300KZ","RB1MZ"};                                 
static ViString s_VBW[] = {"VA","VB1HZ","VB10HZ","VB100HZ","VB1KZ","VB10KZ","VB100KZ","VB1MZ"}; 
static ViString s_sweepTime[] = {"SW","AS"};
static ViString s_sweepMode[] = {"SDW","SN"}; 
static ViString s_All_Coupled[] = {"AL"};

static ViString s_readRBWandVBW[] = {"VB?","RB?","VA?","BA?"};
static ViString s_readSweepTime[] = {"SW?","AS?"}; 
static ViString s_readSweepMode[] = {"SWM?","Normal","Manual","DeltaMkr","Window"};

static ViString s_trigger[] = {"FR","LI","VI","TV","EX","SI"};
static ViString s_readTrigger[] = {"TM?","Free Run","Line","Video","TV_V","External","Single"};
 
static ViString s_detectorMode[] = {"DTN","DTP","DTG","DTS"};
static ViString s_traceAMode[] = {"AW","AV","AB","AM","AG","AG1","AG0"}; 
static ViString s_traceBMode[] = {"BW","BV","BB","BM","BG","BG1","BG0"};
static ViString s_traceMath[] = {"","CH","TR0","TR1","TR2","TR3"};
static ViString s_traceAverage[]  = {"BGR","BGS","BGP","BGC","AGR","AGS","AGP","AGC"};
static ViString s_NormMode[] = {"BNN","BR","BI","BNF","ANN","AR","AI","ANF"}; 

static ViString s_readDetectorMode[] = {"DM?","Normal","Positive","Negative","Sample"};
static ViString s_readTraceStatus[] = {"TB?","TA?"};
static ViString s_readTraceMessage[] = {"B_Write","B_View","B_Blank","B_Norm","B-DL->B","","","B_Max","B_Avg",
                                        "A_Write","A_View","A_Blank","A_norm","A-DL->A","A-B->A","B-A->A","A_Max","A_Avg"};
static ViString s_traceBinary[] = {"TBB?","TBA?"};
static ViString s_traceASCII[] = {"TAB?","TAA?"};

static ViString s_showDL[] = {"DLF","DLN"};
static ViString s_readDisplayLine[] = {"DL?"}; 

static ViString s_positionWindow[]  = {"WTF","WPF","WUL","WLL"};
static ViString s_showWindow[]  = {"WF","WN"};
static ViString s_readMeasWindow[] = {"WLX?","WDX?","WLY?","WDY?","WN?"};

static ViString s_mkrNormal[] = {"MK"};
static ViString s_mkrDelta[] = {"MT","G1?","MPM","UP","DN"};
static ViString s_deleteMarker[] = {"MO"};
static ViString s_mkrSearch[] = {"MN?","MF?","ML?","SF1"};
static ViString s_mkrNDstatus[] = {"markers inactive","only normal marker active","normal and delta markers active"};
static ViString s_softkey[] = {"SF1","SF2","SF3","SF4","SF5","SF6"};
static ViString s_fixedMkrDelta[] = {"FXF","FXN"};
static ViString s_readMKDoptions[] = {"FX?"};  

static ViString s_peakSearch[] = {"PS","NXP","NXL","NXR","NMM","MIS","NXM"}; 
static ViString s_peakTo[] = {"MR","MC"};     
static ViString s_markerDeltaTo[] = {"DS","M1","M3"}; 
                                  
static ViString s_continuousSearch[] = {"CPF","CPN"};
static ViString s_signalTrack[] = {"SGF","SGN"};
static ViString s_peakDelta[] = {"DX","DY"};
static ViString s_readPeakOptions[] = {"CP?","SG?"};   

static ViString s_reset[]={"IP"};
static ViString s_errorQuery[]={"ERRNO?"};
static ViString s_instrRevision[] = {"TYP?"};

static ViString s_calLevel[] = {"CLN","CLF"};                                  
static ViString s_calibration[]  = {"CLA","CLG","IT0","IT1","IT2","IT3","IT4","IT5"};
static ViString s_freqCorrection[] = {"FCF","FCN"};
static ViString s_calCorrection[]  = {"CCF","CCN"};

static ViString s_clearStatusByte[] = {"S2"};

static ViString s_label[] = {"LOF","LON/"};

static ViString s_close[] = {"LC"};

static ViString s_textONOFF[] = {"inactive","active"};
static ViString s_freqUnit[] = {"Hz","kHz","MHz","GHz"};
static ViString s_timeUnit[] = {"ns","us","ms","s"};
static ViString s_levelUnit[] = {"DB","DB","DB","DB","DB","DB","MV"};
static ViString s_positionUnit[] = {"SC","HZ"};

/*****************************************************************************/
/* INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE                               */
/*****************************************************************************/

struct AVR3x61_stringValPair {
    ViStatus stringVal;
    ViString stringName;
};

struct AVR3x61_instrFeatures {
    ViReal64 frequencyLow[3];
    ViReal64 frequencyHigh[3];
    ViReal64 spanLow;
    ViReal64 spanHigh;
    ViReal64 freqOffsetMin;
    ViReal64 freqOffsetMax;
    ViReal64 inputAttMin;
    ViReal64 inputAttMax;
    ViReal64 refLevelMin[7];
    ViReal64 refLevelMax[7];
    ViReal64 refLevelOffsetMin;
    ViReal64 refLevelOffsetMax;
    ViReal64 sweepTimeMini;
    ViReal64 sweepTimeMaxi;
    ViInt16  averageCountMini;
    ViInt16  averageCountMaxi;
    ViInt16  DL_LevelMin;
    ViInt16  DL_LevelMax;
    ViInt16  refUnit;
    ViInt16  span;
    ViInt16  scale;
    ViUInt32 instrumentType;
    ViReal64 deltaXmin;
    ViReal64 deltaXmax;
    ViReal64 deltaYmin;
    ViReal64 deltaYmax;
    ViReal64 calLevelMin;
    ViReal64 calLevelMax;
    ViChar   instrDriverRevision[256];
};

typedef struct AVR3x61_stringValPair AVR3x61_tStringValPair;
    
typedef struct AVR3x61_instrFeatures *AVR3x61_ptrFeatures;

/*****************************************************************************/
/* UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions)                   */
/*****************************************************************************/

ViBoolean AVR3x61_invalidViBooleanRange (ViBoolean val);
ViBoolean AVR3x61_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean AVR3x61_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean AVR3x61_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean AVR3x61_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean AVR3x61_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean AVR3x61_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus AVR3x61_initCleanUp (ViSession openRMSession,ViPSession openInstrSession,
                             ViStatus currentStatus);
ViStatus AVR3x61_defaultInstrSetup (ViSession instrSession, ViUInt32 instrType);
ViStatus AVR3x61_initReset (ViSession, ViUInt32);
ViStatus AVR3x61_Search_Scale (ViSession instrSession, ViPInt16 scale);
ViStatus AVR3x61_Read_DeltaMarker (ViSession instrSession, ViPReal64 position);
                  
/*****************************************************************************/
/* USER-CALLABLE FUNCTIONS (Exportable Functions)                            */
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_init (ViRsrc resourceName, ViBoolean IDquery,
                                ViBoolean reset, ViPSession instrSession)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViSession rmSession = 0;
    ViUInt32 retCnt = 0;
    ViByte Buffer[BUFFER_SIZE];
    ViByte interim[5];
    ViUInt32 AVR3x61_instrType, retCount;
    ViByte cable;
    
    /*- Check input parameter ranges ----------------------------------------*/

    if (AVR3x61_invalidViBooleanRange (IDquery))
        return VI_ERROR_PARAMETER2;
        
    if (AVR3x61_invalidViBooleanRange (reset))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/

    if ((AVR3x61_status = viOpenDefaultRM (&rmSession)) < 0)
        return AVR3x61_status;

    if ((AVR3x61_status = viOpen (rmSession, resourceName, VI_NULL,VI_NULL, instrSession)) < 0) {
        viClose (rmSession);
        return AVR3x61_status;
    }

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    cable = resourceName[0]; /* GPIB or RS-232 cable */
    
    if (cable=='G') { /*GPIB cable*/

        if ((AVR3x61_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 10000)) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
            
        if ((AVR3x61_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
            
        if ((AVR3x61_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                                             VI_FLUSH_ON_ACCESS))   < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
            
        if ((AVR3x61_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                                             VI_FLUSH_ON_ACCESS))   < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
        
        Fmt (Buffer,"%s<%s",s_init[0]);     
        if ((AVR3x61_status = viWrite(*instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
            
    }
    
    else { /*RS-232 cable*/
        /* Serial Interface initialized to 8 databits, 1 stopbit, no parity, */
        /* baud rate = 9600, flow control = XonXoff and timeout = 20000 ms  */
        
        if ((AVR3x61_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_DATA_BITS, 8))<0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
    
        if ((AVR3x61_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_STOP_BITS,VI_ASRL_STOP_ONE))<0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
            
        if ((AVR3x61_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_PARITY, VI_ASRL_PAR_NONE))<0)
            return AVR3x61_status;
        
        if ((AVR3x61_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_BAUD, 9600))<0)
            return AVR3x61_status;
        
        if ((AVR3x61_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_FLOW_CNTRL,VI_ASRL_FLOW_XON_XOFF))<0)
            return AVR3x61_status;                 
        
        if ((AVR3x61_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 20000)) < 0)
            return AVR3x61_status;                     
   
        if ((AVR3x61_status = viSetBuf (*instrSession, VI_ASRL_IN_BUF|VI_ASRL_OUT_BUF, 512)) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
   
        if ((AVR3x61_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
   
        if ((AVR3x61_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,VI_FLUSH_ON_ACCESS))<0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
    
        if ((AVR3x61_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,VI_FLUSH_ON_ACCESS)) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
 
        if ((AVR3x61_status = viSetAttribute (*instrSession, VI_ATTR_TERMCHAR, '\n')) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
 
        if ((AVR3x61_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_END_IN,VI_ASRL_END_TERMCHAR)) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
   
        if ((AVR3x61_status = viSetAttribute (*instrSession, VI_ATTR_ASRL_END_OUT,VI_ASRL_END_NONE)) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
            
        // sends an empty string to the analyzer to init the RS-232 link //
        Fmt (Buffer,"%s<%s\r\n",s_init[3]);    
        if ((AVR3x61_status = viWrite(*instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);    
            
    }       

    /*- Identification Query -------------------------------------------------*/
    
    if (IDquery) {  /* identification executed */
    
        Fmt (Buffer,"%s<%s,%s\r\n",s_init[1],s_init[2]);     
        if ((AVR3x61_status = viWrite(*instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
    
        if ((AVR3x61_status = viRead (*instrSession, Buffer, BUFFER_SIZE, &retCnt)) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
         
        if (Buffer[0] != 'R')  
            return AVR3x61_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
         
        interim[0] = Buffer[1];
        interim[1] = Buffer[2];
        interim[2] = Buffer[3];
        interim[3] = Buffer[4];
        interim[4] = Buffer[(ViInt16)(retCnt)-2];
      
        if (Scan(interim, "%s>%d[b4]", &AVR3x61_instrType)!=1) {
            AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
            return AVR3x61_status;
        }
        
        if (Buffer[5]=='B')  AVR3x61_instrType++;
        
    }
    
    else { /* no identification executed */
        
        Fmt (Buffer,"%s<%s\r\n",s_init[1]);     
        if ((AVR3x61_status = viWrite(*instrSession,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
        
        AVR3x61_instrType = 3261 ;
        
    }   
     
   
    /*- Reset instrument ----------------------------------------------------*/
    if (reset) {
        if ((AVR3x61_status = AVR3x61_initReset (*instrSession,AVR3x61_instrType)) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
    }      
    else { /*- Send Default Instrument Setup ---------------------------------*/ 
        if ((AVR3x61_status = AVR3x61_defaultInstrSetup (*instrSession,AVR3x61_instrType)) < 0)
            return AVR3x61_initCleanUp (rmSession, instrSession, AVR3x61_status);
    }
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function : configure and read                                             */
/* Purpose : application function                                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_configRead(ViSession instrID,ViBoolean trace,ViInt16 peakSearch,
                                     ViInt16 _VI_FAR traceArray[],
                                     ViPReal64 sweepTime,ViChar _VI_FAR sweepUnit[],
                                     ViPReal64 MKRNposition,ViPReal64 MKRNlevel)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    AVR3x61_ptrFeatures featuresPtr;
    ViInt16 status,control;
    ViReal64 sweepValue;
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
       
    if (AVR3x61_invalidViBooleanRange (trace))             
        return VI_ERROR_PARAMETER2;
        
    if (AVR3x61_invalidViInt16Range (peakSearch,0,5))             
        return VI_ERROR_PARAMETER3;
        
    if ((AVR3x61_status = AVR3x61_Read_TraceData (instrID, trace,1,traceArray))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = AVR3x61_Search_Peak (instrID, peakSearch)) <0)
        return AVR3x61_status;   
       
    if ((AVR3x61_status = AVR3x61_Read_SweepTime (instrID, &sweepValue, &control))<0)
        return AVR3x61_status;
       
    if ((AVR3x61_status = AVR3x61_Convert_Time (instrID, sweepValue, sweepTime, sweepUnit))<0)
        return AVR3x61_status;
       
    if ((AVR3x61_status = AVR3x61_Read_NormalMarker (instrID, MKRNposition, MKRNlevel))<0)
        return AVR3x61_status;   
       
    return AVR3x61_status;
} 

/*===========================================================================*/
/* Function: Set Frequency                                                   */           
/* Purpose:  sets up center, start or stop frequency.                        */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_Freq (ViSession instrID, ViInt16 freqType,
                                                       ViReal64 freqValue)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViReal64 span;

    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
        
    if (AVR3x61_invalidViInt16Range (freqType, 0, 2))             
        return VI_ERROR_PARAMETER2;
        
    if (AVR3x61_invalidViReal64Range (freqValue, featuresPtr->frequencyLow[freqType],
                                                 featuresPtr->frequencyHigh[freqType]))             
        return VI_ERROR_PARAMETER3;
        
    Fmt (wrtBuffer,"%s<%s%fHZ\r\n",s_freqType[freqType],freqValue);     
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;
    
    // looks for the span value    
    AVR3x61_Read_FreqSettings (instrID, 1, &span);
    if (span==0) featuresPtr->span=0;
    else featuresPtr->span=1;
                                  
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Frequency Linear Span                                       */           
/* Purpose:  sets up the frequency linear span.                              */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_LinearSpan (ViSession instrID, ViInt16 spanType,
                                                             ViReal64 spanValue)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViInt16 RBWcontrolType;
    ViReal64 RBWvalue;
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;

    if (AVR3x61_invalidViInt16Range (spanType, 0, 2))             
        return VI_ERROR_PARAMETER2;
        
    if (spanType==0) {  /* Manual Span Setting */ 
        if (AVR3x61_invalidViReal64Range (spanValue, featuresPtr->spanLow, 
                                                     featuresPtr->spanHigh))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s,%s%fHZ\r\n",s_spanType[0], s_spanType[spanType + 1],spanValue);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;
    }
    else { /*full span or zero span*/
        Fmt (wrtBuffer,"%s<%s,%s\r\n",s_spanType[0], s_spanType[spanType + 1]);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;
    }
    
    if ((spanType==2)||(spanValue==0)) {
    	featuresPtr->span=0;  /*span null*/
    }	
    else featuresPtr->span=1; /*span not null*/
     
    return AVR3x61_status;
}                         

/*===========================================================================*/
/* Function: Read Frequency Settings                                         */
/* Purpose:  Reads a frequency that you have chosen                          */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_FreqSettings (ViSession instrID, ViInt16 parameter,
                                                                ViPReal64 value)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3x61_invalidViInt16Range (parameter,0,3))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreq[parameter]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 50, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%f",value)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Frequency Log Span                                          */           
/* Purpose:  sets up the frequency logarithmic span.                         */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_LogSpan (ViSession instrID, ViInt16 logStart,
                                                          ViReal64 logStop)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViInt16 RBWcontrolType;
    ViReal64 RBWvalue;
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;

    if (AVR3x61_invalidViInt16Range (logStart, 0, 4))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s,%s%fHZ\r\n",s_logSpan[logStart], s_logSpan[5], logStop);     
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;
    
    featuresPtr->span=1; /*span not null*/
    
    return AVR3x61_status;
}              

/*===========================================================================*/
/* Function: Read Log Span                                                   */
/* Purpose:  Returns the frequency Log Start and Log Stop                    */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_LogSpan (ViSession instrID, ViPReal64 logStart,
                                                           ViPReal64 logStop)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /* reads the frequency Log Start */
    
    Fmt (Buffer,"%s<%s\r\n",s_readLogSpan[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 50, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%f",logStart)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
    
    /* reads the frequency Log Stop */
    
    Fmt (Buffer,"%s<%s\r\n",s_readLogSpan[1]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 50, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%f",logStop)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Frequency Offset                                            */           
/* Purpose:  sets up frequency offset and activates frequency offset         */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_FreqOffset (ViSession instrID, ViBoolean activation,
                                                             ViReal64 freqOffset)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
       
    if (AVR3x61_invalidViBooleanRange (activation))             
        return VI_ERROR_PARAMETER2;   

    if (activation == VI_ON) { /*activation*/
        if (AVR3x61_invalidViReal64Range (freqOffset,featuresPtr->freqOffsetMin,
                                                     featuresPtr->freqOffsetMax))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%fHZ\r\n", s_freqOffset[activation], freqOffset);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;
    }
    else { /*desactivation*/
        Fmt (wrtBuffer,"%s<%s\r\n",s_freqOffset[activation]);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;
    }
     
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Frequency Offset                                           */
/* Purpose:  Reads the frequency offset status and value                     */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_FreqOffset (ViSession instrID, ViPInt16 status,
                                                              ViPReal64 value)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreq[4]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 50, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2x]%f",status,value)!=2) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Frequency Step Size                                         */           
/* Purpose:  sets up and activates or deactivates Frequency Step Size        */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_FreqStepSize (ViSession instrID, ViBoolean controlType,
                                                               ViReal64 freqStepSize)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;

    if (AVR3x61_invalidViBooleanRange (controlType))             
        return VI_ERROR_PARAMETER2;
       
    if (controlType == VI_ON) { /*step size chosen by the analyzer*/ 
        Fmt (wrtBuffer,"%s<%s\r\n", s_freqStepSize[controlType]);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;
    }
    else { /*step size chosen by the user*/   
        if (AVR3x61_invalidViReal64Range (freqStepSize,featuresPtr->spanLow,
                                                       featuresPtr->spanHigh))             
            return VI_ERROR_PARAMETER3;   
        Fmt (wrtBuffer,"%s<%s%fHZ\r\n",s_freqStepSize[controlType],freqStepSize);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;
    }
     
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Frequency Step Size                                        */
/* Purpose:  returns the frequency step size and says whether this parameter */
/*           is controled  by the user or by the analyzer                    */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_FreqStepSize (ViSession instrID, ViPReal64 value,
                                                                ViPInt16 controlType)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreqStepSize[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 50, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%f",value)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readFreqStepSize[1]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
          
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
      
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Frequency Counter                                           */
/* Purpose:  Sets the counter ON / OFF or sets the counter resolution        */
/*===========================================================================*/                               
ViStatus _VI_FUNC AVR3x61_Set_FreqCounter (ViSession instrID, ViBoolean activation,
                                                              ViInt16 resolution)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if (AVR3x61_invalidViBooleanRange (activation))             
        return VI_ERROR_PARAMETER2;
       
    if (AVR3x61_invalidViInt16Range (resolution,0,3))             
        return VI_ERROR_PARAMETER3;
       
    if (activation == VI_ON) {
        Fmt (wrtBuffer,"%s<%s\r\n", s_counterResolution[resolution]);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status; 
    }

    Fmt (wrtBuffer,"%s<%s\r\n", s_counterONOFF[activation]);     
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;

    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Frequency Counter                                          */
/* Purpose:  returns the frequency counter value and says whether the counter*/
/*           is active or inactive                                           */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_FreqCounter (ViSession instrID,ViPReal64 counterValue,
                                                              ViPInt16 counterStatus)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readFreqCounter[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2]", counterStatus)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readFreqCounter[1]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3x61_status;
          
    if (Scan(Buffer,"%s>%f", counterValue)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
      
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Reference Level                                             */           
/* Purpose:  sets up the reference level (Y axis).                           */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_RefLevel (ViSession instrID, ViReal64 refLevel)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViInt16 unit=0;
    ViUInt32 retCount=0;
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
    
    /* sets up the reference level */
    
    if (AVR3x61_invalidViReal64Range (refLevel,featuresPtr->refLevelMin[featuresPtr->refUnit],
                                               featuresPtr->refLevelMax[featuresPtr->refUnit]))             
        return VI_ERROR_PARAMETER2;
        
    if (featuresPtr->refUnit == 6)  refLevel = 1000 * refLevel;    
        
    Fmt (Buffer,"%s<%s%f%s\r\n",s_refLevel[0],refLevel,s_levelUnit[featuresPtr->refUnit]);     
        if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Reference Level                                            */
/* Purpose:  returns the value (without unit) of the reference level         */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_RefLevel(ViSession instrID, ViPReal64 refLevelValue)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readRefLevel[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%f", refLevelValue)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Reference Unit                                              */           
/* Purpose:  Set up the unit of the reference level.                         */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_RefUnit (ViSession instrID, ViInt16 refUnit)
{   
    ViStatus AVR3x61_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
    
    if (AVR3x61_invalidViInt16Range (refUnit,0,4))             
        return VI_ERROR_PARAMETER2;
    
    featuresPtr->refUnit = refUnit;    
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_refUnit[refUnit]);     
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;
       
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Reference Unit                                             */
/* Purpose:  returns the reference unit                                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_RefUnit (ViSession instrID,ViPInt16 unitNumber,
                                                          ViChar _VI_FAR unitName[])
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readRefUnit[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2]", unitNumber)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
    
    Fmt(unitName,"%s<%s", s_readUnitName[*unitNumber]);  
     
    return AVR3x61_status;
} 

/*===========================================================================*/
/* Function: Set Scale                                                       */           
/* Purpose:  Set up the scale of the Y axis .                                */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_Scale (ViSession instrID, ViInt16 scale)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViChar unitName[10];
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;

    if (AVR3x61_invalidViInt16Range (scale,0,10))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_yScale[scale]);     
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;
       
    featuresPtr->scale = scale;
    
    // looks for the reference unit. it may have been modified by the scale change
    AVR3x61_Read_RefUnit (instrID, &featuresPtr->refUnit,unitName); 
                                  
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Scale                                                      */
/* Purpose:  reads the scale : status number and text                        */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_Scale (ViSession instrID, ViPInt16 status,
                                                         ViChar _VI_FAR scaleName[])
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    AVR3x61_ptrFeatures featuresPtr;
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
        
    *status = featuresPtr->scale;
    
    Fmt(scaleName,"%s<%s", s_readScale[*status]);
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Reference Level Offset                                      */           
/* Purpose:  Set up and activates the reference level offset.                */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_RefOffset (ViSession instrID, ViBoolean activation,
                                                            ViReal64 refLevelOffset)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
       
    if (AVR3x61_invalidViBooleanRange (activation))             
        return VI_ERROR_PARAMETER2;   

    if (activation == VI_ON) {  /* activation */
        if (AVR3x61_invalidViReal64Range (refLevelOffset,featuresPtr->refLevelOffsetMin,
                                                         featuresPtr->refLevelOffsetMax))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%fDB\r\n", s_refLevelOffset[activation], refLevelOffset);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;
    }
    else { /* desactivation */
        Fmt (wrtBuffer,"%s<%s\r\n",s_refLevelOffset[activation]);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;
    }
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Reference Level Offset                                     */
/* Purpose:  returns the value (unit is always dB) of the reference level    */
/* offset and says whether this offset is active or not                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_RefLevelOffset (ViSession instrID,ViPReal64 value,
                                                                 ViPInt16 activation)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readRefLevelOffset[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2x]%f",activation, value)!=2) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Input Attenuation                                           */           
/* Purpose:  Set up the Input attenuation.                                   */           
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_Att (ViSession instrID, ViBoolean controlType,
                                                      ViReal64 inputAttValue)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
       
    if (AVR3x61_invalidViBooleanRange (controlType))             
        return VI_ERROR_PARAMETER2;   

    if (controlType == VI_ON) { /* attenuation chosen by the analyzer */ 
        Fmt (wrtBuffer,"%s<%s\r\n", s_att[controlType]);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
             return AVR3x61_status;
    }
    else { /* attenuation chosen by the user */
        if (AVR3x61_invalidViReal64Range (inputAttValue, featuresPtr->inputAttMin,
                                                         featuresPtr->inputAttMax))             
            return VI_ERROR_PARAMETER3; 
        Fmt (wrtBuffer,"%s<%s%fDB\r\n",s_att[controlType],inputAttValue);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;
    }
     
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Attenuation                                                */
/* Purpose:  returns the input attenuation and says whether this parameter   */
/*           is controled by the user or by the analyzer                     */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_Attenuation (ViSession instrID,ViPReal64 value,
                                                              ViPInt16 controlType)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readAtt[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%f", value)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readAtt[1]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
          
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
      
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set RBW                                                         */           
/* Purpose:  Set up the Resolution Bandwidth.                                */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_RBW (ViSession instrID, ViInt16 RBWsetting)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3x61_invalidViInt16Range (RBWsetting,0,10))             
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuffer,"%s<%s\r\n",s_RBW[RBWsetting]);     
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;
       
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set VBW                                                         */           
/* Purpose:  Set up the Video Bandwidth.                                     */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_VBW (ViSession instrID, ViInt16 VBWsetting)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3x61_invalidViInt16Range (VBWsetting,0,7))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_VBW[VBWsetting]);     
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;    

    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read RBW and VBW                                                */
/* Purpose:  reads the resolution or video bandwith and says whether this    */
/*           parameter is controlled by the analyzer or by the user          */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_RBWandVBW (ViSession instrID, ViBoolean parameter,
                                                             ViPReal64 value,
                                                             ViPInt16 controlType)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3x61_invalidViBooleanRange (parameter))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (Buffer,"%s<%s\r\n",s_readRBWandVBW[parameter]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%f", value)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readRBWandVBW[parameter+2]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
        
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    } 
     
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set All Auto Couple                                             */           
/* Purpose:  Sets all couple functions to automatic mode                     */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_All_Auto_Couple (ViSession instrID)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    Fmt (wrtBuffer,"%s<%s\r\n",s_All_Coupled[0]);     
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status; 
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set sweep time                                                  */           
/* Purpose:  Sets the sweep time (sec)                                       */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_SweepTime (ViSession instrID, ViBoolean controlType,
                                                            ViReal64 sweepTime)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
       
    if (AVR3x61_invalidViBooleanRange (controlType))             
        return VI_ERROR_PARAMETER2;
        
    if (controlType == VI_ON) { /*sweep time is chosen by the analyzer*/  
        Fmt (wrtBuffer,"%s<%s\r\n",s_sweepTime[controlType]);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;  
    }
    else { /*sweep time is chosen by the user*/ 
        if (AVR3x61_invalidViReal64Range (sweepTime,featuresPtr->sweepTimeMini,
                                                    featuresPtr->sweepTimeMaxi))             
            return VI_ERROR_PARAMETER3;
        Fmt (wrtBuffer,"%s<%s%fSC\r\n",s_sweepTime[controlType],sweepTime);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;  
    }
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read  Sweep Time                                                */
/* Purpose:  reads the sweep time value and says whether we are in auto or   */
/*           manual control                                                  */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_SweepTime (ViSession instrID, ViPReal64 timeValue,
                                                             ViPInt16 controlType)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readSweepTime[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%f",timeValue)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    Fmt (Buffer,"%s<%s\r\n",s_readSweepTime[1]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2]", controlType)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    } 
     
    return AVR3x61_status;
}   

/*===========================================================================*/
/* Function: Set sweep mode                                                  */           
/* Purpose:  Sets the sweep mode: continuous / single, window sweep on / off */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_SweepMode (ViSession instrID, ViBoolean sweepMode)
{
    ViStatus AVR3x61_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3x61_invalidViBooleanRange (sweepMode))             
        return VI_ERROR_PARAMETER2;
       
    Fmt (wrtBuffer,"%s<%s\r\n",s_sweepMode[sweepMode]);     
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;
     
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Sweep Mode                                                 */
/* Purpose:  returns the sweep Mode: Normal, Manual, Dmkr, Window            */
/*           or Single&Window                                                */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_SweepMode (ViSession instrID, ViPInt16 status,
                                                             ViChar _VI_FAR sweepMode[])
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readSweepMode[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    Fmt(sweepMode,"%s<%s",s_readSweepMode[*status + 1]);
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set trigger                                                     */           
/* Purpose:  Defines the trigger source                                      */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_Trigger(ViSession instrID, ViInt16 source)
{
    ViStatus AVR3x61_status = VI_SUCCESS;  
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3x61_invalidViInt16Range (source,0,5))             
        return VI_ERROR_PARAMETER2;
       
    Fmt (wrtBuffer,"%s<%s\r\n",s_trigger[source]);     
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
       return AVR3x61_status; 
   
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Trigger                                                    */
/* Purpose:  returns the trigger used                                        */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_Trigger (ViSession instrID, ViPInt16 triggerStatus,
                                                           ViChar _VI_FAR triggerName[])
                                                          
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    ViInt16 index=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readTrigger[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2]", triggerStatus)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
    
    Fmt(triggerName,"%s<%s",s_readTrigger[*triggerStatus + 1]);
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Detector Mode                                               */           
/* Purpose:  Specifies detection mode: posi-nega, posi-peak...               */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_TraceDetector (ViSession instrID, ViInt16 detectorMode) 
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3x61_invalidViInt16Range (detectorMode,0,3))             
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuffer,"%s<%s\r\n",s_detectorMode[detectorMode]);   
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    return AVR3x61_status;                                          
}

/*===========================================================================*/
/* Function: Read Detector Mode                                              */
/* Purpose:  returns the detector mode                                       */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_TraceDetector (ViSession instrID, ViPInt16 status,
                                              ViChar _VI_FAR detectorMode[])
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_readDetectorMode[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    Fmt(detectorMode,"%s<%s",s_readDetectorMode[*status+1]); 
    
    return AVR3x61_status; 
}

/*===========================================================================*/
/* Function: Set Trace A Mode                                                */           
/* Purpose:  Defines Trace A mode (Write, View, Max Hold ...)                */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_TraceAMode (ViSession instrID, ViInt16 traceAMode,
                                                             ViInt16 averageCount,
                                                             ViBoolean averageMode)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;

    if (AVR3x61_invalidViInt16Range (traceAMode,0,4))             
        return VI_ERROR_PARAMETER2;
        
    if (traceAMode != 4) { /* set trace mode except trace average */ 
        Fmt (wrtBuffer,"%s<%s\r\n",s_traceAMode[traceAMode]);   
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3x61_status;
    }
    else { /* set trace average: some parameters are needed */
        if (AVR3x61_invalidViInt16Range (averageCount,featuresPtr->averageCountMini,
                                                      featuresPtr->averageCountMaxi))             
            return VI_ERROR_PARAMETER3;
        if (AVR3x61_invalidViBooleanRange (averageMode))             
            return VI_ERROR_PARAMETER4;
        Fmt (wrtBuffer,"%s<%s,%s%d[b2]HZ\r\n",s_traceAMode[5 + averageMode],
                                              s_traceAMode[traceAMode],averageCount);   
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3x61_status;
    }
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Trace B Mode                                                */           
/* Purpose:  Defines Trace B mode : Write, View ...                          */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_TraceBMode (ViSession instrID, ViInt16 traceBMode,
                                                             ViInt16 averageCount,
                                                             ViBoolean averageMode)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;

    if (AVR3x61_invalidViInt16Range (traceBMode,0,4))             
        return VI_ERROR_PARAMETER2;
        
    if (traceBMode != 4) { /* set trace mode except trace average */ 
        Fmt (wrtBuffer,"%s<%s\r\n",s_traceBMode[traceBMode]);   
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3x61_status;
    }
    else { /* set trace average: some parameters are needed */
        if (AVR3x61_invalidViInt16Range (averageCount,featuresPtr->averageCountMini,
                                                      featuresPtr->averageCountMaxi))             
            return VI_ERROR_PARAMETER3;
        if (AVR3x61_invalidViBooleanRange (averageMode))             
            return VI_ERROR_PARAMETER4;
        Fmt (wrtBuffer,"%s<%s,%s%d[b2]HZ\r\n",s_traceBMode[5 + averageMode],
                                              s_traceBMode[traceBMode],averageCount);   
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3x61_status;
    }
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Trace Status                                               */
/* Purpose:  returns the trace A or trace B status                           */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_TraceStatus (ViSession instrID, ViBoolean parameter,
                                                               ViPInt16 status,
                                                               ViChar _VI_FAR message[])
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    
    if (AVR3x61_invalidViBooleanRange (parameter))             
        return VI_ERROR_PARAMETER2; 
 
    Fmt (Buffer,"%s<%s\r\n",s_readTraceStatus[parameter]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
    
    if ( (*status>=10)||(*status==0) )  *status = (*status/10);
    else *status = *status + 6;
    
    Fmt(message,"%s<%s",s_readTraceMessage[*status + 9*parameter]); 
    
    return AVR3x61_status; 
}

/*===========================================================================*/
/* Function: Execute Trace Math                                              */
/* Purpose:  Trace Math execution                                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Execute_TraceMath (ViSession instrID, ViInt16 traceMath)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3x61_invalidViInt16Range (traceMath,0,5))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_traceMath[traceMath]);   
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;   
 
    return AVR3x61_status;  
}

/*===========================================================================*/
/* Function: Start/Stop Trace Average                                        */
/* Purpose:  Starts stops trace average                                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Control_TraceAverage (ViSession instrID, ViBoolean Trace,
                                                                   ViInt16 traceAverage)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3x61_invalidViBooleanRange (Trace))             
        return VI_ERROR_PARAMETER2;
    
    if (AVR3x61_invalidViInt16Range (traceAverage,0,3))             
        return VI_ERROR_PARAMETER3;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_traceAverage[traceAverage + 4*Trace]);   
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;    
        
    return AVR3x61_status;  
}

/*===========================================================================*/
/* Function: Normalizing Mode                                                */
/* Purpose:  Normalize Mode execution                                        */
/*===========================================================================*/ 
ViStatus _VI_FUNC AVR3x61_NormalizingMode (ViSession instrID, ViBoolean trace, ViInt16 mode)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3x61_invalidViBooleanRange (trace))             
        return VI_ERROR_PARAMETER2;
    
    if (AVR3x61_invalidViInt16Range (mode,0,3))             
        return VI_ERROR_PARAMETER3;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_NormMode[mode + 4*trace]);   
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;    
    
    return AVR3x61_status;
} 

/*===========================================================================*/
/* Function: Read Trace Data                                                 */
/* Purpose:  returns trace A or B in an array                                */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_TraceData (ViSession instrID, ViBoolean trace,
                                                             ViBoolean format,
                                                             ViInt16 _VI_FAR traceArray[])
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    AVR3x61_ptrFeatures featuresPtr;
    ViInt16 i;
    ViUInt32 retCount;
    ViByte dataBuffer[1405];
    ViByte wrtBuffer[BUFFER_SIZE];
    
    if (AVR3x61_invalidViBooleanRange (trace))             
        return VI_ERROR_PARAMETER2;
        
    if (AVR3x61_invalidViBooleanRange (format))             
        return VI_ERROR_PARAMETER3;
        
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
       
    for (i=0;i<1405;i++) {
        dataBuffer[i]=0;
    } 
       
    if (format == VI_ON) { /*Binary Transfert*/
        Fmt (wrtBuffer,"%s<%s\r\n",s_traceBinary[trace]);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;
          
        if ((AVR3x61_status = viRead (instrID,dataBuffer,(2*701)+2,&retCount)) < 0)
            return AVR3x61_status;
       
        for (i=0;i<701;i++) {
            traceArray[i]=256*dataBuffer[2*i]+dataBuffer[2*i+1] ;
        }
    }
    
    else { /*ASCII Transfert*/ 
        Fmt (wrtBuffer,"%s<%s\r\n",s_traceASCII[trace]);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;
        for (i=0;i<701;i++) {
            if ((AVR3x61_status = viRead (instrID, dataBuffer, 25, &retCount)) < 0)
                return AVR3x61_status;
            if (Scan(dataBuffer,"%s>%d[b2]", &traceArray[i])!=1) { 
                AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                return AVR3x61_status;
            }   
        }
    }   
     
  return AVR3x61_status; 
}

/*===========================================================================*/
/* Function: Set Display Line                                                */           
/* Purpose:  Specifies display line position and show or hide it             */
/*===========================================================================*/           
ViStatus _VI_FUNC AVR3x61_Set_DisplayLine (ViSession instrID,ViBoolean showDL,
                                                             ViReal64 DL_Level)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;

    if (AVR3x61_invalidViBooleanRange (showDL))             
        return VI_ERROR_PARAMETER2;
    
    if (showDL == VI_ON) { /* activation */
        if (AVR3x61_invalidViReal64Range (DL_Level,featuresPtr->DL_LevelMin,
                                                   featuresPtr->DL_LevelMax))             
            return VI_ERROR_PARAMETER3;
        if (featuresPtr->refUnit == 6) {
        	if (DL_Level < 0) DL_Level = 0; 
        	DL_Level = 1000 * DL_Level;
        }	
        Fmt (wrtBuffer,"%s<%s %f%s\r\n",s_showDL[showDL],DL_Level,
                                        s_levelUnit[featuresPtr->refUnit]);   
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3x61_status;
    }
    else {  /* desactivation */
        Fmt (wrtBuffer,"%s<%s\r\n",s_showDL[showDL]);   
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3x61_status;
    } 
     
    return AVR3x61_status;                                          
}

/*===========================================================================*/
/* Function: Read Display Line                                               */
/* Purpose:  returns the position and the status of the Display Line         */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_DisplayLine(ViSession instrID, ViPReal64 level,
                                                              ViPInt16 status,
                                                              ViChar _VI_FAR message[])
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;

    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
       
    Fmt (Buffer,"%s<%s\r\n",s_readDisplayLine[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 20, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2x]%f", status, level)!=2) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    Fmt(message,"%s<%s",s_textONOFF[*status]);
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Measurement Window                                          */
/* Purpose:  Sets center position, Window width and shows or hides           */
/*           mesurement window                                               */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Set_MeasurementWindow (ViSession instrID,ViBoolean windowStatus,
                                                                   ViReal64 Xposition,
                                                                   ViReal64 Xwidth,
                                                                   ViReal64 Yposition,
                                                                   ViReal64 Ywidth)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViReal64 posstart, posstop, levelmax, levelmin, centerFreq;
    ViUInt32 retCount=0;
    AVR3x61_ptrFeatures featuresPtr;
    ViInt16 offsetStatus;
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
    
    if (AVR3x61_invalidViBooleanRange (windowStatus))             
        return VI_ERROR_PARAMETER2;
        
    if (windowStatus == VI_OFF) {
        Fmt (wrtBuffer,"%s<%s\r\n",s_showWindow[windowStatus]);   
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3x61_status;
    }
    else {
        posstart = Xposition - 0.5*Xwidth;
        posstop = Xposition + 0.5*Xwidth;
        if (posstart < 0) {
        	// reads the center frequency
        	AVR3x61_Read_FreqSettings (instrID, 0, &centerFreq);
        	// changes the center frequency
        	AVR3x61_Set_Freq (instrID, 0, centerFreq - posstart);
    	    // sends the frequency positions
    		Fmt (wrtBuffer,"%s<%s%f%s,%s%f%s\r\n",s_positionWindow[0],0.0,s_positionUnit[featuresPtr->span],
            									  s_positionWindow[1],posstop - posstart,s_positionUnit[featuresPtr->span]);
			if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            	return AVR3x61_status; 
            // changes the center frequency
        	AVR3x61_Set_Freq (instrID, 0, centerFreq);	
        }
        else {
        	Fmt (wrtBuffer,"%s<%s%f%s,%s%f%s\r\n",s_positionWindow[0],posstart,s_positionUnit[featuresPtr->span],
            									  s_positionWindow[1],posstop,s_positionUnit[featuresPtr->span]);
            if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            	return AVR3x61_status;									  
        }    									  
        levelmax=Yposition + 0.5*Ywidth;
        levelmin=Yposition - 0.5*Ywidth;
        if (featuresPtr->refUnit == 6) {
          levelmax = 1000 * levelmax;
          levelmin = 1000 * levelmin;
        }  
        Fmt (wrtBuffer,"%s<%s%f%s,%s%f%s,%s\r\n",s_positionWindow[2],levelmax,s_levelUnit[featuresPtr->refUnit],
             									 s_positionWindow[3],levelmin,s_levelUnit[featuresPtr->refUnit],
             									 s_showWindow[windowStatus]);   
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
            return AVR3x61_status;
    } 
    
    return AVR3x61_status;
}  

/*===========================================================================*/
/* Function: Read Measurement Window                                         */
/* Purpose:  returns the features and the status of the measurement window   */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_MeasurementWindow (ViSession instrID,ViPInt16 status,
                                                                    ViChar _VI_FAR message[],
                                                                    ViPReal64 Xposition,
                                                                    ViPReal64 Xwidth,
                                                                    ViPReal64 Yposition,
                                                                    ViPReal64 Ywidth)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    /*reads the status of the measurement window*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[4]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2]", status)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
    
    Fmt(message,"%s<%s",s_textONOFF[*status]);
    
    /*reads the position of the window along the X-axis*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%f", Xposition)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    /*reads the width of the window along the X-axis*/ 
     
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[1]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%f", Xwidth)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
    
    /*reads the position of the window along the Y-axis*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[2]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%f", Yposition)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    /*reads the width of the window along the Y-axis*/ 
     
    Fmt (Buffer,"%s<%s\r\n",s_readMeasWindow[3]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 25, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%f", Ywidth)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }    
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Marker Status                                              */
/* Purpose:  reads the marker status number -> text                          */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_MarkerStatus (ViSession instrID, ViPInt16 status,
                                                                ViChar _VI_FAR message[])
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_mkrSearch[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
       
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2]",status)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }   
    
    Fmt(message,"%s<%s", s_mkrNDstatus[*status]);  
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Normal Marker                                               */
/* Purpose:  sets the normal marker position                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Set_NormalMarker (ViSession instrID,ViReal64 position)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    AVR3x61_ptrFeatures featuresPtr;
    ViInt16 offsetStatus;
    ViReal64 offsetValue;
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
    
    if ( (position < 0) && (featuresPtr->span == 1) ) {
    
    	// current frequency offset status and value
    	AVR3x61_Read_FreqOffset (instrID, &offsetStatus, &offsetValue);
    	
    	// frequency offset active + frequency offset = 6 GHz
    	AVR3x61_Set_FreqOffset (instrID, 1, 6e9);
    	
    	// sets up the Normal marker
    	position = position - offsetStatus * offsetValue + 6.0e9;
    	Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrNormal[0],position,s_positionUnit[featuresPtr->span]);   
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        	return AVR3x61_status; 
        	
    	// old frequency offset status and value
    	AVR3x61_Set_FreqOffset (instrID, offsetStatus, offsetValue);
   } 	
   else { 
    
    Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrNormal[0],position,s_positionUnit[featuresPtr->span]);   
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;   
        
   }     
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: read normal marker                                              */
/* Purpose:  reads the normal marker position and level (without unit)       */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_NormalMarker (ViSession instrID, ViPReal64 MKRNposition,
                                                                ViPReal64 MKRNlevel)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    AVR3x61_ptrFeatures featuresPtr;
    ViInt16 offsetString1, offsetString2;
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
    
    /*reads the position of the normal marker*/
    
    Fmt (Buffer,"%s<%s\r\n",s_mkrSearch[1]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 50, &retCount)) < 0)
        return AVR3x61_status;
        
    if (featuresPtr->span == 0) {
    	offsetString1 = strcspn(Buffer, "-");
    	offsetString2 = strcspn(Buffer, "E");
    	if ( (offsetString1 == retCount)||(offsetString1 >= offsetString2) ) offsetString1 = 0;
    }
    else {
    	offsetString1 = 0;
    }	
    
    if (Scan(Buffer,"%s[i*]>%f", offsetString1, MKRNposition)!=1) { 
       	AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
       	return AVR3x61_status;
    }
     
    /*reads the level of the normal marker*/ 
     
    Fmt (Buffer,"%s<%s\r\n",s_mkrSearch[2]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 50, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%f", MKRNlevel)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }

    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Delta Marker                                                */
/* Purpose:  Sets the delta marker position                                  */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Set_DeltaMarker (ViSession instrID, ViReal64 position)
{
    ViStatus AVR3x61_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];      
    ViUInt32 retCount=0;
    ViInt16 status, fixedMarkerStatus;
    ViChar message[40];
    ViReal64 freqStart, freqStop, MKRDpos, level, MKRNlevel, MKRNpos;
    AVR3x61_ptrFeatures featuresPtr;
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
        
    AVR3x61_Read_MarkerStatus (instrID, &status, message);
    AVR3x61_Read_DeltaMarkerOptions (instrID, &fixedMarkerStatus, message);
    
    if ( (status == 2)&&(fixedMarkerStatus == 0) ) {  /* normal and delta markers are already active and delta marker is not fixed */
        
        AVR3x61_Read_DeltaMarker(instrID, &MKRDpos);
        MKRDpos = floor (MKRDpos);
        AVR3x61_Delete_Markers (instrID);
        AVR3x61_Set_NormalMarker (instrID, MKRDpos);
        
        Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[0], position, s_positionUnit[featuresPtr->span]);   
    	if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
       		return AVR3x61_status;
     
    }
    
    else if  ( (status == 2)&&(fixedMarkerStatus == 1) ) { /* normal and delta markers are already active and delta marker is fixed */  
    
    	AVR3x61_Read_DeltaMarker (instrID, &MKRDpos);
    	MKRDpos = floor(MKRDpos);
    	AVR3x61_Read_FreqSettings (instrID, 2, &freqStart);
    	AVR3x61_Read_FreqSettings (instrID, 3, &freqStop);
    
    	if ( (MKRDpos > freqStart)&&(MKRDpos < freqStop) ) {
    	
    		Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[0], position, s_positionUnit[featuresPtr->span]);   
    		if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
       			return AVR3x61_status;
    	
    	}
    	else if (MKRDpos == freqStart) {
    	
    		Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[2], position - freqStart, s_positionUnit[featuresPtr->span]);   
    		if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
       			return AVR3x61_status;
       			
       		Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[0], position, s_positionUnit[featuresPtr->span]);   
    		if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
       			return AVR3x61_status;
       			
       		Fmt (wrtBuffer,"%s<%s\r\n",s_mkrDelta[3]);   
    		if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
       			return AVR3x61_status;	
       			
    	}
    	else if (MKRDpos == freqStop) {
    	
       		Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[0], -1.0 * position, s_positionUnit[featuresPtr->span]);   
    		if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
       			return AVR3x61_status;
       		
       		AVR3x61_Read_NormalMarker (instrID, &MKRNpos, &MKRNlevel);
       		
       		Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[2], MKRNpos - position, s_positionUnit[featuresPtr->span]);   
    		if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
       			return AVR3x61_status;
       		
       		Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[0], -1.0 * position, s_positionUnit[featuresPtr->span]);   
    		if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
       			return AVR3x61_status;
       		
       		Fmt (wrtBuffer,"%s<%s\r\n",s_mkrDelta[4]);   
    		if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
       			return AVR3x61_status;
    	}
    
    }
    
    else {
    
    	Fmt (wrtBuffer,"%s<%s%f%s\r\n",s_mkrDelta[0], position, s_positionUnit[featuresPtr->span]);   
    	if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
       		return AVR3x61_status;
    
    }
   
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Delete Markers                                                  */
/* Purpose:  deletes the markers active (normal and/or delta)                */
/*===========================================================================*/                                  
ViStatus _VI_FUNC AVR3x61_Delete_Markers (ViSession instrID)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (Buffer,"%s<%s\r\n",s_deleteMarker[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    return AVR3x61_status;                              
} 

/*===========================================================================*/
/* Function: Search Peak                                                     */
/* Purpose:  set execute peak marker type                                    */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Search_Peak (ViSession instrID, ViInt16 typeSearch)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3x61_invalidViInt16Range (typeSearch,0,6))             
        return VI_ERROR_PARAMETER2;
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_peakSearch[typeSearch]);   
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
       
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: execute Peak to                                                 */
/* Purpose:  peak to setting                                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Execute_PeakTo (ViSession instrID, ViBoolean peakTo)
{
    ViStatus AVR3x61_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3x61_invalidViBooleanRange (peakTo))             
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer,"%s<%s\r\n",s_peakTo[peakTo]);   
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
       
    return AVR3x61_status;
}
 
/*===========================================================================*/
/* Function: Marker Delta to setting                                         */
/* Purpose:  marker delta to setting                                         */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Set_MarkerDeltaTo (ViSession instrID, ViInt16 deltaTo)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;

    if (AVR3x61_invalidViInt16Range (deltaTo,0,2))             
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuffer,"%s<%s\r\n",s_markerDeltaTo[deltaTo]);   
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set Peak Search Options                                         */
/* Purpose:  Sets the features of the peak search                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Set_PeakSearchOptions (ViSession instrID,ViBoolean continuousSearch,
                                                                   ViBoolean signalTrack,
                                                                   ViReal64  deltaX,
                                                                   ViReal64  deltaY)
{
    ViStatus AVR3x61_status = VI_SUCCESS; 
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    AVR3x61_ptrFeatures featuresPtr;
   
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;

    if (AVR3x61_invalidViBooleanRange (continuousSearch))             
        return VI_ERROR_PARAMETER2;
       
    if (AVR3x61_invalidViBooleanRange (signalTrack))             
        return VI_ERROR_PARAMETER3;
    
    if (AVR3x61_invalidViReal64Range(deltaX,featuresPtr->deltaXmin,
                                            featuresPtr->deltaXmax))
        return VI_ERROR_PARAMETER4;
       
    if (AVR3x61_invalidViReal64Range(deltaY,featuresPtr->deltaYmin,
                                            featuresPtr->deltaYmax))
        return VI_ERROR_PARAMETER5;
        
    Fmt (wrtBuffer,"%s<%s,%s,%s%fHZ,%s%fHZ\r\n",s_continuousSearch[continuousSearch],
                                                s_signalTrack[signalTrack],
                                                s_peakDelta[0], deltaX,
                                                s_peakDelta[1], deltaY);
                                             
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
       
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Peak Options                                               */
/* Purpose:  reads the features of the peak search                           */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_Peak_Options (ViSession instrID, ViPInt16 contPeakStatus,
                                                                ViChar _VI_FAR contPeakMessage[],
                                                                ViPInt16 signalTrackStatus,
                                                                ViChar _VI_FAR signalTrackMessage[])
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /*reads the status of the continuous peak search*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readPeakOptions[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2]", contPeakStatus)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
    
    Fmt(contPeakMessage,"%s<%s",s_textONOFF[*contPeakStatus]);
    
    /*reads the status of the signal track*/
    
    Fmt (Buffer,"%s<%s\r\n",s_readPeakOptions[1]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2]", signalTrackStatus)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
    
    Fmt(signalTrackMessage,"%s<%s",s_textONOFF[*signalTrackStatus]);
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Set delta marker options                                        */
/* Purpose:  Sets the display mode and activates or deactivates delta        */
/*           marker fixed and 1/delta marker                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Set_DeltaMarkerOptions (ViSession instrID, ViBoolean fixed)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];      
    ViUInt32 retCount=0;

    if (AVR3x61_invalidViBooleanRange (fixed))             
        return VI_ERROR_PARAMETER3;
       
    Fmt (wrtBuffer,"%s<%s\r\n",s_fixedMkrDelta[fixed]);   
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
          
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Delta Marker Options                                       */
/* Purpose:  returns the options status of the delta marker                  */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_DeltaMarkerOptions (ViSession instrID,
                                                  ViPInt16 fixedMarkerStatus,
                                                  ViChar _VI_FAR fixedMarkerMessage[])
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViUInt32 retCount=0;
    ViByte Buffer[BUFFER_SIZE];
    
    Fmt (Buffer,"%s<%s\r\n",s_readMKDoptions[0]);   
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount))<0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%d[b2]", fixedMarkerStatus)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    Fmt(fixedMarkerMessage,"%s<%s",s_textONOFF[*fixedMarkerStatus]);
    
    return AVR3x61_status; 
}    
    
/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_WriteInstrData (ViSession instrID, ViString writeBuffer)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (wrtBuffer,"%s<%s\r\n",writeBuffer);     
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status; 
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Read Instr Data                                                 */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_readInstrData (ViSession instrID, ViInt32 numBytes,
                                                            ViChar _VI_FAR readBuffer[],
                                                            ViPInt32 bytesRead)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    *bytesRead = 0L;
        
    if ((AVR3x61_status = viRead (instrID, readBuffer, numBytes, bytesRead)) < 0)
        return AVR3x61_status;

    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_reset (ViSession instrSession)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    AVR3x61_ptrFeatures featuresPtr;
    
    if ((AVR3x61_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &featuresPtr)) < 0)
        return AVR3x61_status;
    
    /*  Initialize the instrument to a known state.  */
    Fmt (wrtBuffer,"%s<%s\r\n",s_reset[0]);     
    if ((AVR3x61_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;
   
    if ((AVR3x61_status = AVR3x61_defaultInstrSetup (instrSession, featuresPtr->instrumentType)) < 0)  
        return AVR3x61_status;
        
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. The self test function is not supported by this     */
/*           instrument, and this function returns the warning               */
/*           VI_WARN_NSUP_SELF_TEST. (Self Test not supported)               */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_selfTest (ViSession instrID, ViPInt16 selfTestResult,
                                                       ViChar _VI_FAR selfTestMessage[])
{
    ViStatus AVR3x61_status = VI_WARN_NSUP_SELF_TEST;
    
    Fmt (selfTestMessage, "%s<Self-Test Not Supported");
    *selfTestResult = -1;
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_errorQuery (ViSession instrID, ViPInt32 errorCode,
                                                         ViChar _VI_FAR errorMessage[])
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    
    Fmt (errorMessage, "%s<Error Query Not Supported");
    *errorCode = -1;
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_errorMessage (ViSession instrID, ViStatus errorCode,
                                                           ViChar _VI_FAR errorMessage[])
 
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViInt16 i;
    
    static AVR3x61_tStringValPair statusDescArray[] = 
    {
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_ERROR_PARAMETER1,        "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,        "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,        "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,        "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,        "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,        "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,        "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,        "ERROR: Parameter 8 out of range"},
        {VI_ERROR_INSTR_PARAMETER9 , "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10, "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11, "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12, "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13, "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14, "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15, "ERROR: Parameter 15 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,    "ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE,     "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,  "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE, "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_MARKERINACTIVE,   "ERROR: the chosen multi-marker is inactive"},
        {VI_NULL, VI_NULL}
     };

    AVR3x61_status = viStatusDesc (instrID, errorCode, errorMessage);
    
    if (AVR3x61_status == VI_WARN_UNKNOWN_STATUS) {
        for (i=0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == errorCode) {
                Fmt (errorMessage, "%s<%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        Fmt (errorMessage, "%s<Unknown Error 0x%x[uw8p0]", errorCode); 
        return (VI_WARN_UNKNOWN_STATUS);
    }
    
    AVR3x61_status = VI_SUCCESS;
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_revisionQuery (ViSession instrID, ViChar _VI_FAR driverRev[],
                                                            ViChar _VI_FAR instrRev[])
{
    ViUInt32 retCount = 0;
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    
    Fmt (Buffer,"%s<%s\r\n",s_instrRevision[0]);     
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viRead (instrID, Buffer, 5, &retCount)) < 0)
        return AVR3x61_status;
       
    if (Scan(Buffer,"%s>%s", instrRev)!=1) { 
        AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }
     
    Fmt (driverRev, "%s<%s", AVR3x61_REVISION);
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Calibration                                                     */
/* Purpose:  Set calibration to element                                      */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Calibration (ViSession instrID, ViInt16 element,
                                                          ViReal64 level,
                                                          ViBoolean calCorrection,
                                                          ViBoolean freqCorrection)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViUInt32 retCount=0;
    ViUInt32 timeout;
    
    
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
    
    if (AVR3x61_invalidViInt16Range (element,0,7))             
        return VI_ERROR_PARAMETER2;
        
    if (AVR3x61_invalidViReal64Range (level,featuresPtr->calLevelMin,
                                            featuresPtr->calLevelMax))          
        return VI_ERROR_PARAMETER3;

    if (AVR3x61_invalidViBooleanRange (calCorrection))             
        return VI_ERROR_PARAMETER4;
    
    if (AVR3x61_invalidViBooleanRange (freqCorrection))             
        return VI_ERROR_PARAMETER5;
        
    /* stores the value of the timeout in variable timeout and then disables the timeout */
    if ((AVR3x61_status = viGetAttribute (instrID,VI_ATTR_TMO_VALUE,&timeout)) < 0)
        return AVR3x61_status;
    
    if ((AVR3x61_status = viSetAttribute (instrID, VI_ATTR_TMO_VALUE, 1800000)) < 0)
            return AVR3x61_status;
       
    /* element to calibrate*/  
    Fmt (wrtBuffer,"%s<%s%fDB,%s\r\n",s_calLevel[0],level,s_calibration[element]);     
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;     

    /* calibration and frequency correction */ 
    Fmt (wrtBuffer,"%s<%s,%s\r\n",s_freqCorrection[freqCorrection],
                                  s_calCorrection[calCorrection]);     
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;
    
    /* disables the calibration's signal */
    Fmt (wrtBuffer,"%s<%s\r\n",s_calLevel[1]);     
    if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;  
    
    /*  enables the timeout and sets its value to its last value */  
    if ((AVR3x61_status = viSetAttribute (instrID, VI_ATTR_TMO_VALUE, timeout)) < 0)
        return AVR3x61_status;
        
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Display label                                                   */
/* Purpose:  selects label                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Display_Label (ViSession instrID, ViBoolean showLabel,
                                                            ViString label)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    if (AVR3x61_invalidViBooleanRange (showLabel))             
        return VI_ERROR_PARAMETER2;
       
    if (showLabel == VI_ON) {
        Fmt (wrtBuffer,"%s<%s%s/\r\n",s_label[showLabel],label);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status; 
    }
    else {
        Fmt (wrtBuffer,"%s<%s\r\n",s_label[showLabel]);     
        if ((AVR3x61_status = viWrite(instrID,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
            return AVR3x61_status;
    }
    
    return AVR3x61_status;   
}

/*===========================================================================*/
/* Function : Read Status Byte                                               */
/* purpose : controls the status byte register                               */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Read_StatusByte (ViSession instrID, ViPInt16 statusByte)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViUInt16 status = 0;
    ViByte Buffer[BUFFER_SIZE];
    
    AVR3x61_status = viReadSTB (instrID, &status);
    * statusByte = (ViInt16) status;
    
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function : Clear Status Byte                                              */
/* purpose : clears the status byte register                                 */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Clear_StatusByte (ViSession instrID)
{
 
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    ViByte Buffer[BUFFER_SIZE];
    
    Fmt (Buffer,"%s<%s\r\n",s_clearStatusByte[0]);     
    if ((AVR3x61_status = viWrite(instrID,Buffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;
       
    return AVR3x61_status;
    
}

/*===========================================================================*/
/* Function : Convert Frequency                                              */
/* Purpose : converts the frequency value returned in Hz by the analyzer     */
/*           in the value with the best unit (Hz, kHz, MHz or GHz)           */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Convert_Frequency (ViSession instrID,ViReal64 inputValue,
                                                               ViPReal64 outputValue,
                                                               ViChar _VI_FAR frequencyUnit[])
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViReal64 denom[]={1,1000,1000000,1000000000},interim[3];
    ViInt16 i;
    
    for (i=0;i<3;i++) {
        interim[i] = inputValue / denom[i+1];
    }
    
    i=0; 
    while ( (i<3)&&(fabs(interim[i])>=1) ) {
        i++;
    }
     
    *outputValue = inputValue / denom[i];
    
    Fmt(frequencyUnit,"%s<%s",s_freqUnit[i]); 
    
    return AVR3x61_status;
} 
 
/*===========================================================================*/
/* Function : Convert Time                                                   */
/* Purpose : converts the time value returned in second(s) by the analyzer   */
/*           in the value with the best unit (ns, us, ms or s)               */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_Convert_Time (ViSession instrID, ViReal64 inputValue,
                                                           ViPReal64 outputValue,
                                                           ViChar _VI_FAR timeUnit[])
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViReal64 compare[]={1000000000,1000000,1000,1};
    ViInt16 i;
    
    i=0;
    while ((i<=2)&&(fabs(inputValue)*compare[i]>=1000)) {
        i++;
    }
     
    *outputValue = inputValue*compare[i];
    Fmt(timeUnit,"%s<%s",s_timeUnit[i]);
    
    return AVR3x61_status;
 
}  

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC AVR3x61_close (ViSession instrSession)
{
    AVR3x61_ptrFeatures featuresPtr;
    ViSession rmSession;
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    Fmt (wrtBuffer,"%s<%s\r\n",s_close[0]); // Set Instr to Local Mode    
    if ((AVR3x61_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;

    if ((AVR3x61_status = viGetAttribute (instrSession,VI_ATTR_RM_SESSION,&rmSession))<0)
        return AVR3x61_status;
    if ((AVR3x61_status = viGetAttribute (instrSession,VI_ATTR_USER_DATA,&featuresPtr))<0)
        return AVR3x61_status;
    
    if (featuresPtr != NULL) 
        free (featuresPtr);
    
    AVR3x61_status = viClose (instrSession);
    viClose (rmSession);

    return AVR3x61_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/


/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean AVR3x61_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_OFF && val != VI_ON) ? VI_ON : VI_OFF);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean AVR3x61_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean AVR3x61_invalidViInt32Range  (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean AVR3x61_invalidViUInt16Range  (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range,  the return value is VI_TRUE, otherwise the return    */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean AVR3x61_invalidViUInt32Range  (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}
/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean AVR3x61_invalidViReal32Range  (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean AVR3x61_invalidViReal64Range  (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the AVR3x61_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from AVR3x61_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus AVR3x61_initCleanUp (ViSession openRMSession,
                             ViPSession openInstrSession, ViStatus currentStatus)
{
    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
    
}

/*===========================================================================*/
/* function  Search Scale                                                    */
/* Purpose : search the scale used by the analyzer when the user begin       */
/*           the remote control                                              */
/*===========================================================================*/
ViStatus AVR3x61_Search_Scale (ViSession instrSession, ViPInt16 scale)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    ViInt16   i, unitNumber;
    ViChar unitName[5];
    
    /* returns the unit */
    AVR3x61_Read_RefUnit (instrSession, &unitNumber, unitName);
    
    if (unitNumber != 6) {
    
       // logarithmic scale
       Fmt (Buffer,"%s<%s\r\n",s_yScale[11]);
       if ((AVR3x61_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount))<0)
           return AVR3x61_status;
       if ((AVR3x61_status = viRead (instrSession, Buffer, 5, &retCount)) < 0)
           return AVR3x61_status;
       if (Scan(Buffer,"%s>%d[b2]", scale)!=1) { 
       	   AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
           return AVR3x61_status;
       }
    }  
        
    else {
       // linear scale
       Fmt (Buffer,"%s<%s\r\n",s_yScale[12]);
       if ((AVR3x61_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount))<0)
           return AVR3x61_status;
       if ((AVR3x61_status = viRead (instrSession, Buffer, 5, &retCount)) < 0)
           return AVR3x61_status;
       if (Scan(Buffer,"%s>%d[b2]", scale)!=1) { 
       	   AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
           return AVR3x61_status;
       }
       *scale = *scale + 7; 
    }  
    
    return AVR3x61_status;
} 

/*===========================================================================*/
/* Function : Read Delta Marker												 */
/* Purpose : returns the Delta marker position                               */
/*===========================================================================*/
ViStatus AVR3x61_Read_DeltaMarker (ViSession instrSession, ViPReal64 position)
{

	ViStatus AVR3x61_status = VI_SUCCESS;
	ViUInt32 retCount;
    ViByte Buffer[BUFFER_SIZE];
    ViInt16 x, y, controlType;
	ViReal64 span, start, sweeptime, origin, step;
	AVR3x61_ptrFeatures featuresPtr;
    
    if ((AVR3x61_status = viGetAttribute (instrSession,VI_ATTR_USER_DATA,&featuresPtr)) < 0)
        return AVR3x61_status;
	
	// returns the Delta Marker graphic position
	Fmt (Buffer,"%s<%s\r\n",s_mkrDelta[1]);
    if ((AVR3x61_status = viWrite(instrSession,Buffer,NumFmtdBytes(),&retCount))<0)
    	return AVR3x61_status;
    if ((AVR3x61_status = viRead (instrSession, Buffer, 25, &retCount)) < 0)
        return AVR3x61_status;
    if (Scan(Buffer,"%s>%d[b2x]%d[b2]", &x, &y)!=2) { 
    	AVR3x61_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return AVR3x61_status;
    }    
	
	// converts the graphic position into a physical position
	if (featuresPtr->span == 0) {
	
		AVR3x61_Read_SweepTime (instrSession, &sweeptime, &controlType);
		origin = 0.0;
		step = sweeptime / 700.0; 
		
	}
	else {
	
		AVR3x61_Read_FreqSettings (instrSession, 1, &span);
		AVR3x61_Read_FreqSettings (instrSession, 2, &start);
		origin = start;
		step = span / 700.0; 
	}
	
	*position = origin + x * step;
	
	return AVR3x61_status;
	
}	

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Internal Reset                                                  */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus AVR3x61_initReset (ViSession instrSession,ViUInt32 instrType)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount=0;
    
    /*  Initialize the instrument to a known state.  */
    Fmt (wrtBuffer,"%s<%s\r\n",s_reset[0]);     
    if ((AVR3x61_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
        return AVR3x61_status;
   
    if ((AVR3x61_status = AVR3x61_defaultInstrSetup (instrSession,instrType)) < 0)  
        return AVR3x61_status;
        
    return AVR3x61_status;
}

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the AVR3x61_reset operation and by the    */
/*           AVR3x61_init function if the reset option has not been          */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus AVR3x61_defaultInstrSetup (ViSession instrSession, ViUInt32 instrType)
{
    ViStatus AVR3x61_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    ViByte wrtBuffer[BUFFER_SIZE];
    AVR3x61_ptrFeatures featuresPtr;
    ViInt16 i;
    ViReal64 MKRMpositions[6],positionDelta,MKRMlevels[6];
    ViReal64 levelDelta, span;
    ViChar unitName[10];
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* session and malloc if it has not.                                     */
    if (AVR3x61_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &featuresPtr))
        return AVR3x61_status;
    
    if (featuresPtr == NULL) 
        featuresPtr = malloc (sizeof (struct AVR3x61_instrFeatures));
    
    switch (instrType) {
    
        case 3261 :
            featuresPtr->frequencyLow[0] = 0;
            featuresPtr->frequencyHigh[0] = 2.6e9;
            featuresPtr->frequencyLow[1]= -200e6;
            featuresPtr->frequencyHigh[1] = 2.6e9;
            featuresPtr->frequencyLow[2] = 0;
            featuresPtr->frequencyHigh[2] = 2.8e9;
            featuresPtr->spanLow = 1e3;
            featuresPtr->spanHigh = 2.6e9;
            featuresPtr->sweepTimeMini = 30e-3;
            featuresPtr->sweepTimeMaxi = 1000;
            break;
            
        case 3262 :
            featuresPtr->frequencyLow[0] = 0;
            featuresPtr->frequencyHigh[0] = 3.6e9;
            featuresPtr->frequencyLow[1]= -200e6;
            featuresPtr->frequencyHigh[1] = 3.6e9;
            featuresPtr->frequencyLow[2] = 0;
            featuresPtr->frequencyHigh[2] = 3.8e9;
            featuresPtr->spanLow = 1e3;
            featuresPtr->spanHigh = 3.6e9;
            featuresPtr->sweepTimeMini = 30e-3;
            featuresPtr->sweepTimeMaxi = 1000;
            break;
            
        case 3361 :
            featuresPtr->frequencyLow[0] = 0;
            featuresPtr->frequencyHigh[0] = 2.6e9;
            featuresPtr->frequencyLow[1]= -200e6;
            featuresPtr->frequencyHigh[1] = 2.6e9;
            featuresPtr->frequencyLow[2] = 0;
            featuresPtr->frequencyHigh[2] = 2.8e9;
            featuresPtr->spanLow = 1e3;
            featuresPtr->spanHigh = 2.6e9;
            featuresPtr->sweepTimeMini = 30e-3;
            featuresPtr->sweepTimeMaxi = 1000;
            break;
            
        case 3362 :
            featuresPtr->frequencyLow[0] = 0;
            featuresPtr->frequencyHigh[0] = 3.6e9;
            featuresPtr->frequencyLow[1]= -200e6;
            featuresPtr->frequencyHigh[1] = 3.6e9;
            featuresPtr->frequencyLow[2] = 0;
            featuresPtr->frequencyHigh[2] = 3.8e9;
            featuresPtr->spanLow = 1e3;
            featuresPtr->spanHigh = 3.6e9;
            featuresPtr->sweepTimeMini = 30e-3;
            featuresPtr->sweepTimeMaxi = 1000;
            break;      
        
    }
    
    featuresPtr->instrumentType = instrType;
    featuresPtr->freqOffsetMin = -9.999e9;   /*Hz*/
    featuresPtr->freqOffsetMax = +9.999e9;
    featuresPtr->refLevelOffsetMin = -99.9; /*dB*/
    featuresPtr->refLevelOffsetMax = +99.9;
    featuresPtr->inputAttMin = 0;   /*dB*/
    featuresPtr->inputAttMax = 50;
    featuresPtr->averageCountMini = 2;
    featuresPtr->averageCountMaxi = 1000;
    featuresPtr->DL_LevelMin = -1000;
    featuresPtr->DL_LevelMax = 1000;
    featuresPtr->refLevelMin[0]=-109.9;
    featuresPtr->refLevelMin[1]=-62.9;
    featuresPtr->refLevelMin[2]=-2.9;
    featuresPtr->refLevelMin[3]=3.1;
    featuresPtr->refLevelMin[4]=-19.9;
    featuresPtr->refLevelMin[5]=0;
    featuresPtr->refLevelMin[6]=0.715e-6;
    featuresPtr->refLevelMax[0]=40;
    featuresPtr->refLevelMax[1]=87;
    featuresPtr->refLevelMax[2]=147;
    featuresPtr->refLevelMax[3]=153;
    featuresPtr->refLevelMax[4]=130;
    featuresPtr->refLevelMax[5]=0;
    featuresPtr->refLevelMax[6]=22.4;
    featuresPtr->deltaXmin=1;
    featuresPtr->deltaXmax=700;
    featuresPtr->deltaYmin=1;
    featuresPtr->deltaYmax=400;
    featuresPtr->calLevelMin=-30;
    featuresPtr->calLevelMax=-20;
    
    AVR3x61_Read_RefUnit (instrSession, &featuresPtr->refUnit,unitName);
    
    AVR3x61_Read_FreqSettings (instrSession, 1, &span);
    if (span==0) featuresPtr->span=0;
    else featuresPtr->span=1;
    
    Fmt (featuresPtr -> instrDriverRevision, "%s<%s", AVR3x61_REVISION);    
    
    if (viSetAttribute (instrSession, VI_ATTR_USER_DATA, (ViUInt32)featuresPtr)!= 0)
        return AVR3x61_status;
        
    /* returns the scale used by the spectrum analyzer */
    AVR3x61_Search_Scale (instrSession, &featuresPtr->scale);    
       
    return AVR3x61_status;
    
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
